import { _createSignal, type Signal } from '../state/signal';
import { isFunction } from '../util/types';
import { invoke } from './use-core';
import { useSequentialScope } from './use-sequential-scope';

/**
 * @public
 */
export interface UseSignal {
  <T>(): Signal<T | undefined>;
  <T>(value: T | (() => T)): Signal<T>;
}

/**
 * @public
 */
export const useSignal: UseSignal = <STATE>(initialState?: STATE): Signal<STATE> => {
  const { get, set, iCtx } = useSequentialScope<Signal<STATE>>();
  if (get != null) {
    return get;
  }

  const containerState = iCtx.$renderCtx$.$static$.$containerState$;
  const value = isFunction(initialState) ? invoke(undefined, initialState as any) : initialState;
  const signal = _createSignal(value, containerState, 0, undefined) as Signal<STATE>;
  return set(signal);
};
